#!/bin/bash

# kartmon.sh - monitor any changes in luafiles/ping.txt. If modified,
# write the current time in luafiles/currentutc.txt. 
# 10/19/2023 - cglitcher
# This script is hereby reusable.

# DISCLAIMER: Use this script at your own risk.
# 
# This script is provided as-is, without any express or implied warranties.
# The author(s) disclaim all liability for direct, indirect, or consequential damages
# resulting from your use of this script, including but not limited to loss of data or
# damage to your system. It is your responsibility to review and test the script
# to ensure it meets your requirements before using it in any production environment.
# 
# The author(s) make no guarantees regarding the accuracy, reliability, or 
# suitability of this script for any purpose. Use caution and exercise due diligence 
# when running scripts from untrusted sources.

################################
#     USAGE INSTRUCTIONS       #
################################

# 1. First, install inotify-tools [sudo apt install inotify-tools]
#    - If you are unable to install this, the script will run ok regardless, just with less accuracy
# 2. Place this script in your kart install folder. It can work anywhere, but it's recommended for tidiness
# 3. This script takes one agument: the path to your <kartinstall>/luafiles
#     - this directory is automatically created by kart at runtime
#     - example: /home/srb2kart/.srb2kart/luafiles
# 4. If you are using systemd to control this script, go to linux/NuVoteCoProcess.service for further instructions
#    - else keep reading here
# 5. This script should run in the background. So in your kart launch script, it should appear as follows
#    - [source /path/to/kartmon.sh /path/to/kartinstall/luafiles & srb2kart -server ...]
#    - Replace '/path/to' with the actual path to the file in question
#    - This most important bit is the ampersand (&), as it will put the task in the background
#    - It's ok if there are other commands before and after
# 6. If, for any reason, kart stops, this process will automatically terminate after up to 10 seconds
#    - It is generally ok if more than 1 instance is running, but not desired as it will do all the work twice (or more)!
# 7. To manually kill the process, issue [fg] to bring the process to the foreground, then Ctrl+C it (keyboard interrupt)
#    - If you have more than two of these to kill, keep repeating the above step until there are no more
#    - If you're in a different session/tty, issue [ps aux | grep kartmon] to get the process ID, then [kill <process ID>]

################################
#    END USAGE INSTRUCTIONS    #
################################


# Default values
is_daemon=false

# Parse command-line options
while getopts ":d-:" opt; do
  case $opt in
    d)
      is_daemon=true
      ;;
    -)
      case "${OPTARG}" in
        daemon)
          is_daemon=true
          ;;
        *)
          echo "kartmon: Invalid option: --$OPTARG" >&2
          exit 1
          ;;
      esac
      ;;
    \?)
      echo "kartmon: Invalid option: -$OPTARG" >&2
      exit 1
      ;;
  esac
done

if [ "$is_daemon" = true ]; then
  echo "kartmon: Daemon mode is enabled."
else
  echo "kartmon: Daemon mode is NOT enabled. Add -d or --daemon as your first argument to enable"
fi

# Shift to the next argument after processing options
shift $((OPTIND-1))

# Check if a path is provided as the first or second argument
if [ $# -ge 1 ]; then
  luafiles="$1"
elif [ $# -ge 2 ]; then
  luafiles="$2"
else
  echo "kartmon: Error: Path not provided. Usage: $0 [-d] <path_to_luafiles_dir>"
  echo "kartmon: Note: [-d] is not required. This is to enable daemon mode."
  exit 1
fi

# actual program registers
process_name="srb2kart" # hardcoded for now
file_to_monitor=$luafiles/ping.txt
outfile=$luafiles/currentutc.txt


# Check if ping.txt exists, and if not, create it with "ping" as the content
if [ ! -f $file_to_monitor ]; then
    echo "kartmon: ping" > $file_to_monitor
fi


# Function to check if the process is still running
start_time=$(date +%s)
check_process() {
    # In daemon mode, we don't check if we need to terminate. 
    # That's managed by systemd
    if [ "$is_daemon" = true ]; then
        return
    fi
    
    current_time=$(date +%s)
    elapsed_time=$((current_time - start_time))

    if [ $elapsed_time -ge 10 ]; then
        process_to_monitor=$(pgrep -x $process_name)
        if ! pgrep -x $process_name > /dev/null; then
            echo "kartmon: Process $process_name no longer detected. Exiting script."
            exit 0
        fi
    fi

}

# Trap the SIGTERM signal to exit gracefully
trap 'echo "kartmon: SIGTERM signal received. Exiting script."; exit 0' SIGTERM

if ! command -v inotifywait &> /dev/null; then
    echo "kartmon: Warning: inotifywait NOT detected. Script will continue to function, but it's better with inotify. Try [sudo apt install inotify-tools]"
fi

echo "kartmon: Now monitoring $file_to_monitor, and outputting to $outfile..."
while true; do
    date +%s > $outfile
    # Check if inotifywait exists
    if command -v inotifywait &> /dev/null; then
      if [ "$is_daemon" = true ]; then
        # daemon mode! SAVING cycles
        inotifywait -e modify "$file_to_monitor" -q
      else
        # no daemon mode, we check every 10 seconds
        inotifywait -e modify "$file_to_monitor" -t 10 -q
      fi
    else
        sleep 10
    fi
    check_process
done
